<?php
/**
* ETS - Easy Template System - 2.02
* Copyright (C) 2002  Franck Marcia <phpets@hotmail.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
* @author  Franck Marcia <phpets@hotmail.com>
* @version 2.02
*/

/**
 * Store the content of all template files
 *
 * @var    string
 * @access private
 */
$_ETS_FILES = NULL;

/**
 * Store the content of masks
 *
 * @var    array
 * @access private
 */
$_ETS_BLOCKS = array();

/**
 * Store the content of variable masks
 *
 * @var    array
 * @access private
 */
$_ETS_VARS = array();

/**
 * Store the content of missing variable masks
 *
 * @var    array
 * @access private
 */
$_ETS_MISSINGS = array();

/**
 * Error handler
 *
 * @param  string  Message to print out
 * @return NULL
 * @access private
 */
function _ets_error($message, $verbose)
{
	if ($verbose && (error_reporting() & E_WARNING)) {
		echo "<br>\n<b>ETS warning:</b> $message in <b>" . $GLOBALS['HTTP_SERVER_VARS']['PATH_TRANSLATED'] . "</b><br>\n";
	}
	return NULL;
}

/**
 * Read template files
 *
 * @param  string  Comma separated list of file names
 * @return void
 * @access private
 */
function _ets_file($filenames)
{
	global $_ETS_FILES;

	// Construct an array of file names
	if (!is_array($filenames)) {
		$filenames = explode(',', $filenames);
	}

	// Then, an array of file content
	$content = array();
	foreach($filenames as $filename) {

		$filename = trim($filename);

		if ($handle = @fopen($filename, 'rb')) {
			$size = filesize($filename);
			$content[$filename] = @fread($handle, $size);
			fclose($handle);

		} else {
			_ets_error("file $filename not found", TRUE);
		}
	}

	// Store the contents
	$_ETS_FILES = implode('', $content);
}

/**
 * Take off comments in a mask
 *
 * @param  string  Mask content
 * @return string  Mask content without comments
 * @access private
 */
function _ets_nocomment($maskcontent)
{
	return preg_replace('/\{\*.*\*\}/smU', '', $maskcontent);
}

/**
 * Extract the content of a mask from files content
 *
 * @param  mixed   Comma separated list (string) or array of file names
 * @param  boolean Flag that indicates how to manage errors
 * @return string  Content of the mask or NULL on failure
 * @access private
 */
function _ets_mask($maskname, $verbose)
{
	global $_ETS_FILES, $_ETS_BLOCKS;

	// First, search in the cache
	if (isset($_ETS_BLOCKS[$maskname])) {
		return (FALSE === $_ETS_BLOCKS[$maskname]) ? NULL : $_ETS_BLOCKS[$maskname];
	}

	// Then try to find out the mask from files content
	if (preg_match('/\{mask:' . $maskname . '\}(.*)\{\/mask:' . $maskname . '\}/sm', $_ETS_FILES, $m)) {
		// Replace all nested masks by their corresponding placeholder
		$m[1] = preg_replace('/\{mask:([a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*)\}.*\{\/mask:\1\}/sm', '{$1}', $m[1]);
		$m[1] = preg_replace('/\{mis:([a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*)\}.*\{\/mis:\1\}/sm', '{$1_missing}', $m[1]);
		$m[1] = $maskcache[$maskname] = _ets_nocomment(preg_replace('/\{var:([a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*)\}.*\{\/var:\1\}/sm', '{$1}', $m[1]));
		return $m[1];
	}

	// Nothing worked
	_ets_error("mask $maskname not found", $verbose);

	// Failure
	$maskcache[$maskname] = FALSE;
	return NULL;
}

/**
 * Build a variable
 *
 * @param  mixed   Variable data
 * @param  string  Variable name
 * @return string  Built variable
 * @access private
 */
function _ets_var($vardata, $varname)
{
	global $_ETS_FILES, $_ETS_VARS;

	// Missing variables
	if (!isset($vardata) || 0 == strlen($vardata)) {
		return NULL;
	}

	// Search in the cache
	if (isset($_ETS_VARS[$varname])) {
		return (FALSE === $_ETS_VARS[$varname]) ? NULL : str_replace('{value}', $vardata, $_ETS_VARS[$varname]);
	}

	// Try to find out the mask from files content
	if (preg_match('/\{var:' . $varname . '\}(.*)\{\/var:' . $varname . '\}/sm', $_ETS_FILES, $m)) {
		// Nested masks
		$m[1] = _ets_nocomment(preg_replace('/\{mask:([a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*)\}.*\{\/mask:\1\}/sm', '{$1}', $m[1]));
		$_ETS_VARS[$varname] = $m[1];
		return str_replace('{value}', $vardata, $_ETS_VARS[$varname]);
	}

	// No mask
	return $vardata;
}

/**
 * Build an missing variable
 *
 * @param  string  Variable name
 * @return string  Built variable
 * @access private
 */
function _ets_missing($varname)
{
	global $_ETS_FILES, $_ETS_MISSINGS;

	// First, search in the cache
	if (isset($_ETS_MISSINGS[$varname])) {
		return (FALSE === $_ETS_MISSINGS[$varname]) ? NULL : $_ETS_MISSINGS[$varname];
	}

	// Then try to find out the mask from files content
	if (preg_match('/\{mis:' . $varname . '\}(.*)\{\/mis:' . $varname . '\}/sm', $_ETS_FILES, $m)) {
		// Nested masks
		$m[1] = _ets_nocomment(preg_replace('/\{mask:([a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*)\}.*\{\/mask:\1\}/sm', '{$1}', $m[1]));
		return $_ETS_MISSINGS[$varname] = $m[1];
	}

	// No mask
	$_ETS_MISSINGS[$varname] = FALSE;
	return NULL;
}

/**
 * Build a level
 *
 * @param  object  Level data
 * @param  string  Level name
 * @param  string  Current mask name
 * @param  boolean Flag that indicates how to manage errors
 * @return string  Built level
 * @access private
 */
function _ets_level($leveldata, $levelname, $maskname, $verbose)
{
	// Try to find out a MASK property as mask...
	if (isset($leveldata->MASK)) {
		$mask = _ets_mask($leveldata->MASK, $verbose);

	// ... or try to find out a STRING property as mask...
	} elseif (isset($leveldata->STRING)) {
		$mask = _ets_nocomment($leveldata->STRING);

	// ... or use given mask name
	} elseif (isset($maskname)) {
		$mask = _ets_mask($maskname, $verbose);

	// ... or search for a mask which has the same name as the level
	} else {
		$mask = _ets_mask($levelname, $verbose);
	}

	// Iterate thought properties...
	$tags = $tagnames = array();
	foreach ($leveldata as $tagname => $tagdata) {

		$tagnames[$tagname] = '{' . $tagname . '}';

		// ... which can be an array...
		if (is_array($tagdata)) {
			$tags[$tagname] = _ets_array($tagdata, $tagname, NULL, $verbose);

		// ... or an object...
		} elseif (is_object($tagdata)) {
			$tags[$tagname] = _ets_level($tagdata, $tagname, NULL, $verbose);

		// ... or a scalar value
		} else {
			$tags[$tagname] = _ets_var($tagdata, $tagname);
			if (!isset($tags[$tagname])) {
				$tags[$tagname . '_missing'] = _ets_missing($tagname);
				$tagnames[$tagname . '_missing'] = '{' . $tagname . '_missing}';
			}
		}

	}

	// Done
	return str_replace($tagnames, $tags, $mask);
}

/**
 * Build a array of levels
 *
 * @param  array   Array of levels
 * @param  string  Array name
 * @param  string  Current mask name
 * @param  boolean Flag that indicates how to manage errors
 * @return string  Built level
 * @access private
 */
function _ets_array($arraydata, $arrayname, $maskname, $verbose)
{
 	// Use given mask name or a mask which has the same name as the array
	if (!isset($maskname)) {
		$maskname = $arrayname;
 	}

	// Iterate thought elements of the array...
	$levels = array();
	foreach ($arraydata as $levelname => $leveldata) {

		// ... which can be an array...
		if (is_array($leveldata)) {
			$levels[$levelname] = _ets_array($leveldata, $levelname, $maskname, $verbose);

		// ... or an object...
		} elseif (is_object($leveldata)) {
			$levels[$levelname] = _ets_level($leveldata, $levelname, $maskname, $verbose);

		// ... but nothing else
		} else {
			_ets_error("level $levelname of array $arrayname is neither an object nor an array", $verbose);
		}
	}

	// Done
	return implode('', $levels);
}

/**
 * Build a template
 *
 * @param  mixed   Tree to be parsed (array or object)
 * @param  string  Comma separated list of file names
 * @return string  Built template
 * @access public
 */
function sprintt($datatree, $filenames = NULL)
{
	// Retrieve file contents
	if (isset($filenames)) {
		_ets_file($filenames);
	}

	// The root can be a array...
	if (is_array($datatree)) {
		$built = _ets_array($datatree, 'main', NULL, TRUE);

	// ... or an object...
	} elseif (is_object($datatree)) {
		$built = _ets_level($datatree, 'main', NULL, TRUE);

	// ... but nothing else
	} else {
		return _ets_error('the root level is neither an object nor an array', TRUE);
	}

	// Search for php masks then parse them
	while (preg_match('/\{php\}(.*)\{\/php\}/smU', $built, $phpmask)) {
		$built = str_replace($phpmask[0], eval('return ' . $phpmask[1] . ';'), $built);
	}

	// Search for constant masks and clean up unused tags
	$loop = 0;
	while (preg_match_all('/(\{([a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*)\})/', $built, $mask)) {
		foreach($mask[1] as $maskname => $masktag) {
			$built = str_replace($masktag, _ets_mask($mask[2][$maskname], FALSE), $built);
		}
		// Beyond 100 loops, consider it's a recursive construct (dirty)
		if (100 < ++$loop) {
			_ets_error('too many loops in constant masks process (recursive construct?)', TRUE);
			break;
		}
	}

	// Done
	return $built;
}

/**
 * Print out a built template
 *
 * @param  mixed   Tree to be parsed (array or object)
 * @param  string  Comma separated list of file names
 * @return void
 * @access public
 */
function printt($datatree, $filenames = NULL)
{
	echo sprintt($datatree, $filenames);
}

?>